SuperStrict

Rem
bbdoc: iB3D: Blitz3D-style command set for Irrlicht.Core
End Rem
Module Irrlicht.B3D

ModuleInfo "Name: iB3D: Blitz3D-style command set for Irrlicht.Core"
ModuleInfo "Description: "
ModuleInfo "License: Public Domain"
ModuleInfo "Authors: gman, _Jen, Wave (Delta), CoderLaureate (THashtable), Yan (Bit functions), eax for SViewFrustum::classifyBoxRelation()"
ModuleInfo "CustomAttr: "

Import Irrlicht.Core
Import BRL.LinkedList
Import BRL.Map
Import "b3d.cpp"

Include "hashtable.bmx"
Include "global.bmx" 
Include "draw.bmx"
Include "entity.bmx"
Include "camera.bmx"
Include "texture.bmx"
Include "light.bmx"
Include "pivot.bmx"
Include "plane.bmx"
Include "surface.bmx"
Include "mesh.bmx"
Include "brush.bmx"
Include "terrain.bmx"
Include "collision.bmx"
Include "collision_detect.bmx"
Include "collision_response.bmx"
Include "picking.bmx"
Include "delta.bmx"
Include "primitive.bmx"
Include "font.bmx"

Rem
bbdoc: Base type for all ib3d types
EndRem
Type ib3d_base
EndType

Extern
	Function _RotateAnimMesh(pIAnimatedMesh:Int,pIMeshManipulator:Int,rx:Float,ry:Float,rz:Float)="RotateAnimMesh"
	Function _RotateMesh(pIMesh:Int,pIMeshManipulator:Int,rx:Float,ry:Float,rz:Float)="RotateMesh"
	' Tests If a aabox is within the Frustum.
   	' \Return 0 = Whole box is outside the frustum, 1 = Box intersects with frustum, 2 = Whole box is inside the frustum.
	' by eax: http://irrlicht.sourceforge.net/phpBB2/viewtopic.php?t=3997   	
	Function SViewFrustum_classifyBoxRelation:Int(pSViewFrustum:Int,pAABBox3df:Int)
EndExtern

Rem
bbdoc: Rounds a float to a certain # of digits
returns: the rounded float
EndRem
Function round:Float(val:Float,dec:Int)
	val:*(10^dec)
	Local temp:Int=Int(val)
	If val-temp>=.5 Then temp=temp+1
	Return Float(temp/(10^dec))
EndFunction

Rem
bbdoc: Retruns a float trimmed to a certain # of decimal places
returns: the center point as a Vector3df
about: bRound rounds the float before trimming it.
EndRem
Function FloatToFixedPoint:String(f:Float,decimals:Int=3,bRound:Int=True)
	If bRound Then f=round(f,decimals)
	Local i:Long = (10^decimals)*f
	Local value:String = String.fromlong(i)
	
	If value.length<=decimals
		Return "0."+(RSet("",decimals-value.length)).Replace(" ","0")+value
	Else
		Return value[0..value.length-decimals] + "." + value[value.length-decimals..value.length]  
	EndIf
EndFunction

Rem
bbdoc: Returns the center point of a triangle based on 3 vertices
returns: the center point as a Vector3df
about: based on forum code from big10p
EndRem
Function TriangleCenter:Vector3df(pointA:Vector3df,pointB:Vector3df,pointC:Vector3df)
	Local tx:Float=pointB.getX()+((pointC.getX()-pointB.getX())/2)
	Local ty:Float=pointB.getY()+((pointC.getY()-pointB.getY())/2)
	Local tz:Float=pointB.getZ()+((pointC.getZ()-pointB.getZ())/2)	
	Local cvx:Float=tx-((tx-pointA.getX())/3)
	Local cvy:Float=ty-((ty-pointA.getY())/3)
	Local cvz:Float=tz-((tz-pointA.getZ())/3)
	Return _VECTOR3DF(cvx,cvy,cvz)
EndFunction

Rem

x0,y0,z0 = vert 0 coords.
x1,y1,z1 = vert 1 coords.
x2,y2,z2 = vert 2 coords.

; Calculate And add a lone 'centre of tri'
; vertex (needed For per-tri rotations).
tx# = x1 + ((x2-x1)/2)
ty# = y1 + ((y2-y1)/2)
tz# = z1 + ((z2-z1)/2)
cvx# = tx - ((tx-x0)/3)
cvy# = ty - ((ty-y0)/3)
cvz# = tz - ((tz-z0)/3)
AddVertex(d_surf,cvx,cvy,cvz)
EndRem

Function Vector3df_Precision_EqEq:Int(vect1:Vector3df,vect2:Vector3df,decimals:Int=3)
	Local tempVect1:Vector3df=_VECTOR3DF(round(vect1.getX(),decimals),round(vect1.getY(),decimals),round(vect1.getZ(),decimals))
	Local tempVect2:Vector3df=_VECTOR3DF(round(vect2.getX(),decimals),round(vect2.getY(),decimals),round(vect2.getZ(),decimals))
	Return tempVect1.EqEq(tempVect2)
EndFunction

' NOTE: SViewFrustum will give a BB of the current view.


' from Yan
' http://www.blitzmax.com/Community/posts.php?topic=56607
Function CheckBit%(value%, bit%)
  Return (value & (1 Shl bit)) Shr bit
End Function

Function SetBit%(value% Var, bit%)
  value :| (1 Shl bit)
End Function

Function ClearBit%(value% Var, bit%)
  value :& ~(1 Shl bit)
End Function

Function ToggleBit%(value% Var, bit%)  
  value :~ (1 Shl bit)
End Function

Rem
NOTE: this doesnt take rotation into consideration
from: http://www.blitzbasic.com/codearcs/codearcs.php?code=792
Function SphereBoxIntersection(sx#,sy#,sz#,sr#,bx#,by#,bz#,bw#,bh#,bd#)

	; sx#,sy#,sz# = sphere x,y,z centre co-ordinates
	; sr# = sphere radius
	; bx#,by#,bz# = box x,y,z corner co-ordinates (center point)
	; bw#,bh#,bd# = box width,height,depth

	Local dmin#=0
	Local sr2#=sr*sr
	
	; x axis
	If sx < bx

		dmin=dmin+((sx-bx)*(sx-bx))
		
	Else If sx>(bx+bw)

		dmin=dmin+(((sx-(bx+bw)))*((sx-(bx+bw))))

	EndIf

	; y axis
	If sy < by

		dmin=dmin+((sy-by)*(sy-by))
		
	Else If sy>(by+bh)

		dmin=dmin+(((sy-(by+bh)))*((sy-(by+bh))))

	EndIf

	; z axis
	If sz < bz

		dmin=dmin+((sz-bz)*(sz-bz))

	Else If sz>(bz+bd)

		dmin=dmin+(((sz-(bz+bd)))*((sz-(bz+bd))))

	EndIf

	If dmin#<=sr2# Then Return True Else Return False

End Function
EndRem

Rem
; ID: 947
; Author: elias_t
; Date: 2004-02-27 11:18:29
; Title: Ray-&gt;Sphere Intersection
; Description: Returns intersection points
; http://www.blitzbasic.com/codearcs/codearcs.php?code=947

;infinite ray to sphere intersection example.
;The intersection points are returned.

Dim picked1#(2);1st intersection point
Dim picked2#(2);2nd intersection point [optional]


; x1,y1,z1    : 1st point of segment
; x2,y2,z2    : 2nd point of segment
; x3,y3,z3, r : coordinates And radius of sphere)

Function sphere_line_intersection (x1#,y1#,z1#, x2#,y2#,z2#, x3#,y3#,z3#,r#)

Local mu#

Local dx1#=(x2-x1)
Local dy1#=(y2-y1)
Local dz1#=(z2-z1)

Local dx2#=(x1-x3)
Local dy2#=(y1-y3)
Local dz2#=(z1-z3)


Local a# = dx1*dx1 + dy1*dy1 + dz1*dz1
Local b# = 2*( dx1*dx2 + dy1*dy2 + dz1*dz2 )
Local c# = (x3*x3)+(y3*y3)+(z3*z3)+(x1*x1)+(y1*y1)+(z1*z1)-2*(x3*x1+y3*y1+z3*z1)-(r*r)


Local bm#=(b * b)
Local da#=2*a
Local fac#=(4 * a * c)

Local i#=bm - fac

Local sqi#=Sqr(i)



;if segment crosses the sphere
If i>=0

  ;incoming intersection
  mu = (-b - sqi) / da
  picked1(0) = x1 + mu*dx1
  picked1(1) = y1 + mu*dy1
  picked1(2) = z1 + mu*dz1

  ;
  ;outgoing intersection
  ;remove this if you only need the first intersection point
  mu = (-b + sqi) / da
  picked2(0) = x1 + mu*dx1
  picked2(1) = y1 + mu*dy1
  picked2(2) = z1 + mu*dz1
  ;

Return 1
EndIf



End Function
EndRem

