' entity functions converted to use Irrlicht nodes
' TODO: the globl params for the entityX/yaw/position/rotate line of functions needs checked.  think need to do something with
' transforming using the parents transformation.
' TODO: ib3d_EntityName -> can be set in modelling program
' TODO: ib3d_EntityParent -> globl param
' TODO: ib3d_EntityTexture frame parameter
' TODO: implement animated texture support
' TODO: ib3d_PointEntity does not function yet
' TODO: any functions that work with materials (ie EntityAmbientColor) should maybe work on all materials by default?  change to -1 default and use setMaterial().
'
'
Rem
' code to determine forward vector
irr::core::vector3df vector = irr::core::vector3df(0,0,1);
irr::core::matrix4 m;
m.setRotationDegrees(node->GetRotation());
m.transformVect(vector);
EndRem

Public

Type ib3d_Entity Extends ib3d_base

	' maps a b3d entity to an ISceneNode
	Global SceneNodeToEntity:THashtable=Null
	Global EntityList:TList=Null

	Field _userdata:TMap=Null
	Field _node:ISceneNode=Null
	Field _radius:Vector3df=Null
	Field _collision_offset:Vector3df=Vector3df.create()
	Field _lastposition:Vector3df=Null
	Field _speed:Float=1.0
	Field _pickmode:Int=PM_UNPICKABLE

	' event handlers
	Field _OnCollision:ONCOLLISIONEVENTHANDLER=Null
	Field _OnPicked:ONPICKEDEVENTHANDLER=Null
		
	Method New()
		' setup the ISceneNode to b3d_entity map
		If Not SceneNodeToEntity Then SceneNodeToEntity=New THashtable
		If Not EntityList Then EntityList=New TList
		EntityList.AddLast(Self)
	EndMethod

	Method Destroy()
		SceneNodeToEntity.RemoveKey(_node.handle)
		EntityList.Remove(Self)

		' clear this from the last picked if set
		If _g_ib3d_pick_manager.getLastPickedEntity()=Self Then _g_ib3d_pick_manager.setLastPickedEntity()

		' remove from pickable list
		If _g_ib3d_pick_manager.PickableList And _pickmode<>PM_UNPICKABLE Then _g_ib3d_pick_manager.PickableList.Remove(Self)
		
		If _userdata Then _userdata.Clear()
		
		' remove from the scene graph
		_node.Remove()
		_node=Null
	EndMethod

	Method setNode(node:ISceneNode)
		If Not node.IsValid() Then RuntimeError "setNode is invalid"
		SceneNodeToEntity.Add(node.handle,Self,True)
		_node=node
		_node.setRotation(_VECTOR3DF(0,0,0))
		_node.setPosition(_VECTOR3DF(0,0,0))
		_node.updateAbsolutePosition()
		' set the initial collision radius based on the bounding box extents/2
		setCollisionRadiusFromBox()
		' change the ID from the default -1 to 0.  this needed for picking.
		'_node.setID(0)		
	EndMethod
	
	Function findEntity:ib3d_entity(node:ISceneNode)
		If Not node Or Not node.IsValid() Then Return Null
		Local temp:TKeyValuePair=SceneNodeToEntity.ContainsKey(node.handle)
		If Not temp Return Null Else Return ib3d_entity(temp.Value)
	EndFunction

	Method saveID()
		setUserData("id_restore",String.FromInt(_node.getID()))
	EndMethod

	Method restoreID()
		_node.setID(String(getUserData("id_restore")).ToInt())
		removeUserData("id_restore")
	EndMethod

	' ----------------------------------------------
	' userdata methods
	' ----------------------------------------------
	
	Method setUserData(key:String,data:Object)
		If Not _userdata Then _userdata=New TMap
		_userdata.Insert(key,data)
	EndMethod
	
	Method getUserData:Object(key:String)
		If _userdata Then Return _userdata.ValueForKey(key) Else Return Null
	EndMethod
	
	Method removeUserData(key:String)
		If _userdata Then _userdata.Remove(key)
	EndMethod		

	Method setSpeed(new_speed:Float=0)
		_speed=new_speed
	EndMethod
	
	Method getSpeed:Float()
		Return _speed
	EndMethod

	' ----------------------------------------------
	' picking support methods/functions
	' ----------------------------------------------
	
	Method setPickMode(mode:Int=PM_UNPICKABLE,obscurer:Int=True)		
		' PM_SPHERE is not supported.  default to PM_BOX.
		If mode=PM_SPHERE Or Not MESH(Self) Then mode=PM_BOX

		_pickmode=mode
		
		If _pickmode=PM_UNPICKABLE Then
			PICK_MANAGER.PickableList.Remove(Self)
		Else
			PICK_MANAGER.PickableList.AddLast(Self)
		EndIf
	EndMethod
	
	Method getPickMode:Int()
		Return _pickmode
	EndMethod

	Method setOnPickedEventHandler(handler:ONPICKEDEVENTHANDLER=Null)
		_OnPicked=handler
	EndMethod
	
	Method OnPicked(pickTimeMs:Int,intersection:Vector3df=Null,triangle:Triangle3df=Null)
		If _OnPicked Then _OnPicked.OnPicked(Self,pickTimeMs,intersection,triangle)
	EndMethod

	' ----------------------------------------------
	' collision support methods
	' ----------------------------------------------

	Method setCollisionOffset(offset:Vector3df)
		_collision_offset=offset
	EndMethod
	
	Method setCollisionRadius(x:Float,y:Float,z:Float)
		_radius=Vector3df.createFromVals(x,y,z)
	EndMethod
	
	Method setCollisionRadiusFromBox(box:AABbox3df=Null)
		If Not box Then box=_node.getBoundingBox()
		Local boxextent:Vector3df=box.getExtent()
		setCollisionRadius(boxextent.getX()/2,boxextent.getY()/2,boxextent.getZ()/2)		
	EndMethod

	Method getCollisionRadius:Vector3df()
		Return _radius
	EndMethod
	
	Method setOnCollisionEventHandler(handler:ONCOLLISIONEVENTHANDLER)
		_OnCollision=handler
	EndMethod
	
	Method OnCollision(collision:COLLISION)
		If _OnCollision Then _OnCollision.OnCollision(collision)
	EndMethod
	
	Method setLastPosition(pos:Vector3df=Null)
		If Not _node Or Not _node.isValid() Then RuntimeError "Node is invalid"
		If Not pos Then _lastposition=_node.getPosition() Else _lastposition=pos
	EndMethod
	
	Method getLastPosition:Vector3df()
		Return _lastposition
	EndMethod
	
	' ---------------------------------------------------------------
	' B3D methods
	' ---------------------------------------------------------------
			
	Method X:Float(globl:Int=False)
		If Not _node Or Not _node.isValid() Then RuntimeError "Node is invalid"
		If (globl=False)
			Return _node.getPosition().getX()
		Else
			Return _node.getAbsolutePosition().getX()
		EndIf
	EndMethod

	Method Y:Float(globl:Int=False)
		If Not _node Or Not _node.isValid() Then RuntimeError "Node is invalid"
		If (globl=False)
			Return _node.getPosition().getY()
		Else
			Return _node.getAbsolutePosition().getY()
		EndIf
	EndMethod

	Method Z:Float(globl:Int=False)
		If Not _node Or Not _node.isValid() Then RuntimeError "Node is invalid"
		If (globl=False)
			Return _node.getPosition().getZ()
		Else
			Return _node.getAbsolutePosition().getZ()
		EndIf
	EndMethod

	Method Pitch:Float(globl:Int=False)
		If Not _node Or Not _node.isValid() Then RuntimeError "Node is invalid"
		If (globl=False)
			Return _node.getRotation().getX()
		Else
			Return _node.getAbsoluteTransformation().getRotationDegrees().getX()
		EndIf
	EndMethod

	Method Yaw:Float(globl:Int=False)
		If Not _node Or Not _node.isValid() Then RuntimeError "Node is invalid"
		If (globl=False)
			Return _node.getRotation().getY()
		Else
			Return _node.getAbsoluteTransformation().getRotationDegrees().getY()
		EndIf
	EndMethod

	Method Roll:Float(globl:Int=False)
		If Not _node Or Not _node.isValid() Then RuntimeError "Node is invalid"
		If (globl=False)
			Return _node.getRotation().getZ()
		Else
			Return _node.getAbsoluteTransformation().getRotationDegrees().getZ()
		EndIf
	EndMethod
	
	Method Scale(x:Float,y:Float,z:Float)
		If Not _node Or Not _node.isValid() Then RuntimeError "Node is invalid"
		_node.setScale(Vector3df.createFromVals(x,y,z))
	EndMethod
	
	Method Show()
		If Not _node Or Not _node.isValid() Then RuntimeError "Node is invalid"
		_node.setVisible(True)
	EndMethod
	
	Method Hide()
		If Not _node Or Not _node.isValid() Then RuntimeError "Node is invalid"
		_node.setVisible(False)
	EndMethod

	Method Name(name:String)
		If Not _node Or Not _node.isValid() Then RuntimeError "Node is invalid"
		_node.setName(name)
	EndMethod
	
	Method getName:String()
		If Not _node Or Not _node.isValid() Then RuntimeError "Node is invalid"
		Return _node.getName()
	EndMethod
	
	' TODO: implement globl param
	Method Parent(parent:ib3d_Entity,globl:Int=True)
		If Not _node Or Not _node.isValid() Then RuntimeError "Node is invalid"
		_node.setParent(parent._node)
	EndMethod
	
	Method GetParent:ib3d_Entity()
		If Not _node Or Not _node.isValid() Then RuntimeError "Node is invalid"
		Return findEntity(_node)
	EndMethod

	Method CountChildren:Int()
		If Not _node Or Not _node.isValid() Then RuntimeError "Node is invalid"
		Return _node.getChildren().Dimensions()[0]
	EndMethod
	
	Method GetChild:ib3d_Entity(index:Int)
		If Not _node Or Not _node.isValid() Then RuntimeError "Node is invalid"
		Local children:ISceneNode[]=_node.getChildren()
		Local childcnt:Int=children.Dimensions()[0]
		If childcnt<1 Or index>childcnt Then RuntimeError "Child index "+index+" outside of child count "+childcnt
	
		Return findEntity(children[index-1])
	EndMethod
	
	Method FindChild:ib3d_Entity(name:String)
		If Not _node Or Not _node.isValid() Then RuntimeError "Node is invalid"
		Local child:ISceneNode=Null
		Local children:ISceneNode[]=_node.getChildren()
		Local childcnt:Int=children.Dimensions()[0]
		
		If childcnt>0
			Local tempnode:ISceneNode
			For Local i:Int=0 To childcnt-1
				tempnode=children[i]
				If tempnode.getName()=name
					child=tempnode
					Exit
				EndIf		
			Next
		EndIf

		If Not child Then Return Null Else Return findEntity(child)
	EndMethod

	' ----------------------------------------------------------------------
	' not sure about the following
	' ----------------------------------------------------------------------

	' TODO: not sure about global
	Method Position(x:Float,y:Float,z:Float,globl:Int=False)
		If Not _node Or Not _node.isValid() Then RuntimeError "Node is invalid"
		_node.setPosition(Vector3df.createFromVals(x,y,z))
	EndMethod
	
	Method Rotate(pitch:Float,yaw:Float,roll:Float,globl:Int=False)
		If Not _node Or Not _node.isValid() Then RuntimeError "Node is invalid"

		' limit to 360 degrees
		Rem
		If pitch>360 Then pitch=pitch-(Int(pitch/360)*360)
		If yaw>360 Then yaw=yaw-(Int(yaw/360)*360)
		If roll>360 Then roll=roll-(Int(roll/360)*360)
		If pitch<0 Then pitch=360+pitch
		If yaw<0 Then yaw=360+yaw
		If roll<0 Then roll=360+roll
		EndRem
		
		_node.setRotation(_VECTOR3DF(pitch,yaw,roll))		
	EndMethod
	
	Method Move(x:Float,y:Float,z:Float)
		If Not _node Or Not _node.isValid() Then RuntimeError "Node is invalid"

		' multiply in the engine speed
		If x<>y Then x:*_g_ib3d_engine.getSpeed()
		If y<>0 Then y:*_g_ib3d_engine.getSpeed()
		If z<>0 Then z:*_g_ib3d_engine.getSpeed()

		' multiply in the entity speed
		If x<>y Then x:*getSpeed()
		If y<>0 Then y:*getSpeed()
		If z<>0 Then z:*getSpeed()
		
		Rem
		' multiply in the delta time			
		If x<>y Then x:*Delta.Time()
		If y<>0 Then y:*Delta.Time()
		If z<>0 Then z:*Delta.Time()
		EndRem
		
		' from: http://irrlicht.sourceforge.net/phpBB2/viewtopic.php?t=4680
		Local dest:Vector3df=Vector3df.createFromVals(x,y,z)
		_node.getRelativeTransformation().transformVect(dest)
		_node.setPosition(dest)
	EndMethod
	
	Method Turn(pitch:Float,yaw:Float,roll:Float,globl:Int=False)
		If Not _node Or Not _node.isValid() Then RuntimeError "Node is invalid"

		' multiply in the engine speed
		If pitch<>0 Then pitch:*_g_ib3d_engine.getSpeed()
		If yaw<>0 Then yaw:*_g_ib3d_engine.getSpeed()
		If roll<>0 Then roll:*_g_ib3d_engine.getSpeed()

		' multiply in the entity speed
		If pitch<>0 Then pitch:*getSpeed()
		If yaw<>0 Then yaw:*getSpeed()
		If roll<>0 Then roll:*getSpeed()

		Rem
		If pitch<>0 Then pitch:*Delta.Time()
		If yaw<>0 Then yaw:*Delta.Time()
		If roll<>0 Then roll:*Delta.Time()
		EndRem

		' from: http://irrlicht.sourceforge.net/phpBB2/viewtopic.php?t=4680
		Local m:Matrix4=Matrix4.create()
		Local n:Matrix4=Matrix4.create()
		Local t:Matrix4=Matrix4.create()
		
		m.setRotationDegrees(_node.getRotation())
		
		n.setRotationDegrees(_VECTOR3DF(0,0,roll))
		t.setRotationDegrees(_VECTOR3DF(pitch,0,0))
		n.MultEq(t)
		t.setRotationDegrees(_VECTOR3DF(0,yaw,0))
		n.MultEq(t)
			
	    m.MultEq(n)
		_node.setRotation(m.getRotationDegrees())
		' ---------------------------

		'_node.setRotation(_node.getRotation().Plus(Vector3df.createFromVals(pitch,yaw,roll)))
	EndMethod

	Method Translate(x:Float,y:Float,z:Float)
		If Not _node Or Not _node.isValid() Then RuntimeError "Node is invalid"
		
		' multiply in the engine speed
		If x<>y Then x:*_g_ib3d_engine.getSpeed()
		If y<>0 Then y:*_g_ib3d_engine.getSpeed()
		If z<>0 Then z:*_g_ib3d_engine.getSpeed()

		' multiply in the entity speed
		If x<>y Then x:*getSpeed()
		If y<>0 Then y:*getSpeed()
		If z<>0 Then z:*getSpeed()

		Rem
		' multiply in the delta time
		If x<>0 Then x:*Delta.Time()
		If y<>0 Then y:*Delta.Time()
		If z<>0 Then z:*Delta.Time()
		EndRem
		
		Local m:Matrix4=Matrix4.create()
		m.setTranslation(Vector3df.createFromVals(x,y,z))
		Local pos:Vector3df=_node.getPosition()
		m.translateVect(pos)
		_node.setPosition(pos)		
	EndMethod
	
	' TODO: implement
	Method Point(target:ib3d_Entity,roll:Float=0.0)
		If Not _node Or Not _node.isValid() Then RuntimeError "Node is invalid"
		PointToVect(target._node.getPosition())
	EndMethod
	
	' from: based on code by Edwart on Irrlicht forums
	' http://irrlicht.sourceforge.net/phpBB2/viewtopic.php?t=18781
	Method PointToVect:Vector3df(target:Vector3df)

		'Local rot:Vector3df=_node.getRotation()
		'Local roll:Float=roll()

		Local direction:Vector3df=target.Minus(_node.getPosition())
		'DebugLog(direction.getX()+" | "+direction.getY()+" | "+direction.getZ())
		'DebugLog(direction.getHorizontalAngle().getX()+" | "+direction.getHorizontalAngle().getY()+" | "+direction.getHorizontalAngle().getZ())
		'If direction.getHorizontalAngle()
		_node.setRotation(direction.getHorizontalAngle())

		'Self.Turn(0,0,roll)
	EndMethod
	
	Method GetForwardVector:Vector3df(length:Float=1.0)
		Local vector:Vector3df=_VECTOR3DF(0,0,length)
		Local m:Matrix4=Matrix4.create()
		m.setRotationDegrees(_node.GetRotation())
		m.transformVect(vector)
		Return vector
	EndMethod	
	
	' TODO: note index only support 0-3
	' TODO: implement animated texture support
	Method setTexture(texture:TEXTURE,frame:Int=0,index:Int=0)
		If Not _node Or Not _node.isValid() Then RuntimeError "Node is invalid"
		
		_node.setMaterialTexture(index,texture.tex)

	EndMethod
	
	Method setDebugDataVisible(bShow:Int)
		If Not _node Or Not _node.isValid() Then RuntimeError "Node is invalid"
		_node.setDebugDataVisible(bShow)
	EndMethod

   	Method SetColor(red:Float,green:Float,blue:Float,MaterialIDX:Int=0)
		If Not _node Or Not _node.isValid() Then RuntimeError "Node is invalid"
		Local color:SColor=SColor.createFromVals(0,red,green,blue)
		
		_node.getMaterial(MaterialIDX).getAmbientColor().set(color.getAlpha(),color.getRed(),color.getGreen(),color.getBlue())
		_node.getMaterial(MaterialIDX).getDiffuseColor().set(color.getAlpha(),color.getRed(),color.getGreen(),color.getBlue())
   	EndMethod

	Method setAmbientColor(red:Float,green:Float,blue:Float,MaterialIDX:Int=0)
		If Not _node Or Not _node.isValid() Then RuntimeError "Node is invalid"
		Local color:SColor=SColor.createFromVals(0,red,green,blue)
		
		_node.getMaterial(MaterialIDX).getAmbientColor().set(color.getAlpha(),color.getRed(),color.getGreen(),color.getBlue())
	EndMethod

	Method setDiffuseColor(red:Float,green:Float,blue:Float,MaterialIDX:Int=0)
		If Not _node Or Not _node.isValid() Then RuntimeError "Node is invalid"
		Local color:SColor=SColor.createFromVals(0,red,green,blue)
		
		_node.getMaterial(MaterialIDX).getDiffuseColor().set(color.getAlpha(),color.getRed(),color.getGreen(),color.getBlue())
	EndMethod
	
	Method setEmissiveColor(red:Float,green:Float,blue:Float,MaterialIDX:Int=0)
		If Not _node Or Not _node.isValid() Then RuntimeError "Node is invalid"
		Local color:SColor=SColor.createFromVals(0,red,green,blue)
		
    	_node.getMaterial(MaterialIDX).getEmissiveColor().set(color.getAlpha(),color.getRed(),color.getGreen(),color.getBlue())
	EndMethod

	Method setSpecularColor(red:Float,green:Float,blue:Float,MaterialIDX:Int=0)
		If Not _node Or Not _node.isValid() Then RuntimeError "Node is invalid"
		Local color:SColor=SColor.createFromVals(0,red,green,blue)
		
    	_node.getMaterial(MaterialIDX).getSpecularColor().set(color.getAlpha(),color.getRed(),color.getGreen(),color.getBlue())
	EndMethod														

	Method setFX(E_MATERIAL_FLAG:Int,bNewVal:Int,MaterialIDX:Int=-1)
		If Not _node Or Not _node.isValid() Then RuntimeError "Node is invalid"
		If MaterialIDX<0 Then
			_node.setMaterialFlag(E_MATERIAL_FLAG,bNewVal)
		Else
			_node.getMaterial(MaterialIDX).setFlag(E_MATERIAL_FLAG,bNewVal)
		EndIf
	EndMethod

	Method SetBlend(E_MATERIAL_TYPE:Int,MaterialIDX:Int=-1)
		If Not _node Or Not _node.isValid() Then RuntimeError "Node is invalid"
		If MaterialIDX<0 Then
			_node.setMaterialType(E_MATERIAL_TYPE)
		Else
			_node.getMaterial(MaterialIDX).setMaterialType(E_MATERIAL_TYPE)
		EndIf
	EndMethod

	Method setShininess(shininess:Float=0.0,MaterialIDX:Int=0)
		If Not _node Or Not _node.isValid() Then RuntimeError "Node is invalid"
		_node.getMaterial(MaterialIDX).setShininess(shininess)
	EndMethod
	
	Method getDistanceFrom:Double(entity:ib3d_Entity)
		If Not _node Or Not _node.isValid() Then RuntimeError "Node is invalid"
		If Not entity._node Or Not entity._node.isValid() Then RuntimeError "Distance node is invalid"
		Return _node.getPosition().getDistanceFrom(entity._node.getPosition())
	EndMethod
												
EndType

Rem
bbdoc: <description>
returns: <what it returns>
about: <i>Parameters</i>
<ul>
<li> - </li>
</ul>
<details>
keyword: <keyword>
EndRem

' ----------------------------------------------------------------------
' done
' ----------------------------------------------------------------------

Rem
bbdoc: Translates (moves) an entity relative to its current position and not its orientation
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
<li>x:Float - x amount the entity will be translated by</li>
<li>y:Float - y amount the entity will be translated by</li>
<li>z:Float - z amount the entity will be translated by</li>
</ul>
EndRem
Function ib3d_TranslateEntity(entity:ib3d_Entity,x:Float,y:Float,z:Float,globl:Int=False)
	If entity=Null Then Throw "Entity does not exist" 
	entity.Translate(x,y,z)
EndFunction

Rem
bbdoc: Frees up resources associated with the entity and removes it from the scene
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
</ul>
EndRem
Function ib3d_FreeEntity(entity:ib3d_Entity Var)
	If entity=Null Then Throw "Entity does not exist" 
	entity.Destroy()
	entity=Null
EndFunction

Rem
bbdoc: The X-coordinate of the entity
returns: The X-coordinate
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
<li>globl:Int - Pass true to get the coordinate in world coordinates, False to get coordinates in relation to the parent.  Default is False.</li>
</ul>
NOTE: If the entity has no parent then local and global coordinates are the same. <br>
In this case you can think of the 3d world as the parent. <br><br>
Global coordinates refer to the 3d world. Blitz 3D uses a left-handed system: <br><br>
X+ is to the right <br>
Y+ is up <br>
Z+ is forward ( into the screen ) <br><br>
Every entity also has its own Local coordinate system. <br><br>
The global system never changes. <br>
But the local system is carried along as an entity moves and turns. <br><br>
This same concept is used in the entity movement commands: <br><br>
ib3d_MoveEntity(entity,0,0,1)<br><br>
No matter what the orientation this moves one unit forward. 
EndRem
Function ib3d_EntityX:Float(entity:ib3d_Entity,globl:Int=False)
	If entity=Null Then Throw "Entity does not exist" 
	Return entity.X(globl)
EndFunction

Rem
bbdoc: The Y-coordinate of the entity
returns: The Y-coordinate
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
<li>globl:Int - Pass true to get the coordinate in world coordinates, False to get coordinates in relation to the parent.  Default is False.</li>
</ul>
NOTE: If the entity has no parent then local and global coordinates are the same. <br>
In this case you can think of the 3d world as the parent. <br><br>
Global coordinates refer to the 3d world. Blitz 3D uses a left-handed system: <br><br>
X+ is to the right <br>
Y+ is up <br>
Z+ is forward ( into the screen ) <br><br>
Every entity also has its own Local coordinate system. <br><br>
The global system never changes. <br>
But the local system is carried along as an entity moves and turns. <br><br>
This same concept is used in the entity movement commands: <br><br>
ib3d_MoveEntity(entity,0,0,1)<br><br>
No matter what the orientation this moves one unit forward. 
EndRem
Function ib3d_EntityY:Float(entity:ib3d_Entity,globl:Int=False)
	If entity=Null Then Throw "Entity does not exist" 
	Return entity.Y(globl)
EndFunction

Rem
bbdoc: The Z-coordinate of the entity
returns: The Z-coordinate
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
<li>globl:Int - Pass true to get the coordinate in world coordinates, False to get coordinates in relation to the parent.  Default is False.</li>
</ul>
NOTE: If the entity has no parent then local and global coordinates are the same. <br>
In this case you can think of the 3d world as the parent. <br><br>
Global coordinates refer to the 3d world. Blitz 3D uses a left-handed system: <br><br>
X+ is to the right <br>
Y+ is up <br>
Z+ is forward ( into the screen ) <br><br>
Every entity also has its own Local coordinate system. <br><br>
The global system never changes. <br>
But the local system is carried along as an entity moves and turns. <br><br>
This same concept is used in the entity movement commands: <br><br>
ib3d_MoveEntity(entity,0,0,1)<br><br>
No matter what the orientation this moves one unit forward. 
EndRem
Function ib3d_EntityZ:Float(entity:ib3d_Entity,globl:Int=False)
	If entity=Null Then Throw "Entity does not exist" 
	Return entity.Z(globl)
EndFunction

Rem
bbdoc: Returns the Pitch (X) angle of an entity
returns: The current Pitch
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
<li>globl:Int - Pass true if the returned value should be relative to 0 rather than the parent entity.  Default is False.</li>
</ul>
The pitch angle is the X angle.<br>
NOTE: this value differs from B3D proper.
EndRem
Function ib3d_EntityPitch:Float(entity:ib3d_Entity,globl:Int=False)
	If entity=Null Then Throw "Entity does not exist" 
	Return entity.Pitch(globl)
EndFunction

Rem
bbdoc: Returns the Yaw (Y) angle of an entity
returns: The current Yaw
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
<li>globl:Int - Pass true if the returned value should be relative to 0 rather than the parent entity.  Default is False.</li>
</ul>
The yaw angle is the Y angle<br>
NOTE: this value differs from B3D proper.
EndRem
Function ib3d_EntityYaw:Float(entity:ib3d_Entity,globl:Int=False)
	If entity=Null Then Throw "Entity does not exist" 
	Return entity.Yaw(globl)
EndFunction

Rem
bbdoc: Returns the Roll (Z) angle of an entity
returns: The current Roll
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
<li>globl:Int - Pass true if the returned value should be relative to 0 rather than the parent entity.  Default is False.</li>
</ul>
The roll angle is the Z angle<br>
NOTE: this value differs from B3D proper.
EndRem
Function ib3d_EntityRoll:Float(entity:ib3d_Entity,globl:Int=False)
	If entity=Null Then Throw "Entity does not exist" 
	Return entity.Roll(globl)
EndFunction

Rem
bbdoc: Scales an entity so that it is of an absolute size
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
<li>x:Float - x size of entity</li>
<li>y:Float - y size of entity</li>
<li>z:Float - z size of entity</li>
</ul>
Scale values of 1,1,1 are the default size when creating/loading entities. <br><br>
Scale values of 2,2,2 will double the size of an entity. <br><br>
Scale values of 0,0,0 will make an entity disappear. <br><br>
Scale values of less than 0,0,0 will invert an entity and make it bigger. <br><br>
EndRem
Function ib3d_ScaleEntity(entity:ib3d_Entity,x:Float,y:Float,z:Float)
	If entity=Null Then Throw "Entity does not exist" 
	entity.Scale(x,y,z)
EndFunction

Rem
bbdoc: Shows an entity
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
</ul>
Once an entity has been hidden using ib3d_HideEntity, use show entity to make it visible and involved in collisions again. Note that ib3d_ShowEntity has no effect if the enitities parent object is hidden. <br><br>
Entities are shown by default after creating/loading them, so you should only need to use ib3d_ShowEntity after using ib3d_HideEntity. <br><br>
ib3d_ShowEntity affects the specified entity only - child entities are not affected. <br><br>
EndRem
Function ib3d_ShowEntity(entity:ib3d_Entity)
	If entity=Null Then Throw "Entity does not exist" 
	entity.Show()
EndFunction

Rem
bbdoc: Hides an entity so that it is no longer visible and is no longer involved in collisions
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
</ul>
The main purpose of hide entity is to allow you to create entities at the beginning of a program, hide them, then copy them and show as necessary in the main game. This is more efficient than creating entities mid-game. <br><br>
If you wish to hide an entity so that it is no longer visible but still involved in collisions, then use EntityAlpha 0 instead. This will make an entity completely transparent. <br><br>
HideEntity affects the specified entity and all of its child entities, if any exist. <br><br>
EndRem
Function ib3d_HideEntity(entity:ib3d_Entity)
	If entity=Null Then Throw "Entity does not exist" 
	entity.Hide()
EndFunction

Rem
bbdoc: Sets an entities name
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
<li>name:String - name of entity</li>
</ul>
EndRem
Function ib3d_NameEntity(entity:ib3d_Entity,name:String)
	If entity=Null Then Throw "Entity does not exist" 
	entity.Name(name)
EndFunction

Rem
bbdoc: Returns the name of an entity
returns: Entity name
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
</ul>
EndRem
Function ib3d_EntityName:String(entity:ib3d_Entity)
	If entity=Null Then Throw "Entity does not exist" 
	Return entity.GetName()
EndFunction

Rem
bbdoc: Attaches an entity to a parent
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
<li>parent:ib3d_Entity - parent entity</li>
<li>globl:Int - True for the child entity to maintain its global position and orientation.  Defaults to True.</li>
</ul>
<details>
keyword: <keyword>
EndRem
Function ib3d_EntityParent(entity:ib3d_Entity,parent:ib3d_Entity,globl:Int=True)
	If entity=Null Then Throw "Entity does not exist" 
	entity.Parent(parent,globl)
EndFunction

Rem
bbdoc: Returns an entities parent
returns: Parent entity
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
</ul>
EndRem
Function ib3d_GetParent:ib3d_Entity(entity:ib3d_Entity)
	If entity=Null Then Throw "Entity does not exist" 
	Return entity.GetParent()
EndFunction

Rem
bbdoc: Returns the # of children of an entity
returns: # of children
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
</ul>
EndRem
Function ib3d_CountChildren:Int(entity:ib3d_Entity)
	If entity=Null Then Throw "Entity does not exist" 
	Return entity.CountChildren()
EndFunction

Rem
bbdoc: Returns a child of an entity
returns: a child entity or null if none found
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
<li>index:int - index of child entity.  Should be in range of 1...ib3d_CountChildren(entity) inclusive</li>
</ul>
EndRem
Function ib3d_GetChild:ib3d_Entity(entity:ib3d_Entity,index:Int)
	If entity=Null Then Throw "Entity does not exist" 
	Return entity.GetChild(index)	
EndFunction

Rem
bbdoc: Returns the first child of the specified entity with the matching child name
returns: a child entity or null if none found
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
<li>name:String - name of the child to find within the entity</li>
</ul>
EndRem
Function ib3d_FindChild:ib3d_Entity(entity:ib3d_Entity,name:String)
	If entity=Null Then Throw "Entity does not exist" 
	Return entity.FindChild(name)
EndFunction

Rem
bbdoc: Applies a texture to an entity
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
<li>frame:Int - frame of texture.  Default to 0.</li>
<li>index:Int - index # of texture.  Should be in the range of 0-3.  Defaults to 0.</li>
</ul>
NOTE: frame and animated textures are not currently supported.
EndRem
Function ib3d_EntityTexture(entity:ib3d_Entity,texture:TEXTURE,frame:Int=0,index:Int=0)
	If entity=Null Then Throw "Entity does not exist" 
	entity.setTexture(texture,frame,index)	
EndFunction

Rem
bbdoc: Shows debug data such as boundingboxes
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
<li>bShowDebugData:Int - True if to show, False to hide.  Default is False.
</ul>
Not all entities support this feature.
EndRem
Function ib3d_EntityDebugDataVisible(entity:ib3d_Entity,bShowDebugData:Int)
	If entity=Null Then Throw "Entity does not exist" 
	entity.setDebugDataVisible(bShowDebugData)
EndFunction

' ----------------------------------------------------------------------
' ----------------------------------------------------------------------
' not sure about
' ----------------------------------------------------------------------

' -----------------------------------------------
' Entity UserData
' TODO: note new functions
' -----------------------------------------------

Rem
bbdoc: Associates an object with the entity.
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
<li>key:String - Key used to access the user data</li>
<li>data:Object - data to store in the entity.  Passing Null will remove the user data.</li>
</ul>
EndRem
Function ib3d_EntitySetUserData(entity:ib3d_Entity,key:String,data:Object)
	If Not entity Then Throw "Entity does not exist" 
	If Not data Then 
		entity.removeUserData(key)
	Else
		entity.setUserData(key,data)
	EndIf
EndFunction

Rem
bbdoc: Retrieves user data stored with the entity.
returns: a user data object or Null if not found
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
<li>key:String - Key for the user data</li>
</ul>
EndRem
Function ib3d_EntityGetUserData:Object(entity:ib3d_Entity,key:String)
	If Not entity Then Throw "Entity does not exist" 
	Return entity.getUserData(key)
EndFunction

Rem
bbdoc: Removes user data stored with the entity
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
<li>key:String - Key for the user data</li>
</ul>
EndRem
Function ib3d_EntityRemoveUserData(entity:ib3d_Entity,key:String)
	If Not entity Then Throw "Entity does not exist" 
	entity.removeUserData(key)
EndFunction

' -----------------------------------------------
' Entity Movement
' -----------------------------------------------

Rem
bbdoc: Positions an entity at an absolute position in 3D space
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
<li>x:Float - x coordinate the entity will be positioned at</li>
<li>y:Float - y coordinate the entity will be positioned at</li>
<li>z:Float - z coordinate the entity will be positioned at</li>
<li>globl:Int - True if the position should be relative to 0,0,0 rather than the entities parent position. Defaults to False.</li>
</ul>
Entities are positioned using an x,y,z coordinate system. x, y and z each have their own axis, and each axis has its own set of values. By specifying a value for each axis, you can position an entity anywhere in 3D space. 0,0,0 is the centre of 3D space, and if the camera is pointing in the default positive z direction, then positioning an entity with a z value of above 0 will make it appear in front of the camera, whereas a negative z value would see it disappear behind the camera. Changing the x value would see it moving sideways, and changing the y value would see it moving up/down. <br><br>
Of course, the direction in which entities appear to move is relative to the position and orientation of the camera. 
EndRem
Function ib3d_PositionEntity(entity:ib3d_Entity,x:Float,y:Float,z:Float,globl:Int=False)
	If entity=Null Then Throw "Entity does not exist" 
	entity.Position(x,y,z,globl)
EndFunction

Rem
bbdoc: Rotates an entity so that it is at an absolute orientation.
returns: <what it returns>
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
<li>pitch:Float - angle in degress of pitch rotation</li>
<li>yaw:Float - angle in degrees of yaw rotation</li>
<li>roll:Float - angle in degrees of roll rotation</li>
<li>globl:Int - True if the angle to be rotated should be relative to 0,0,0 rather than a parent entities orientation. Defaults to False.</li>
</ul>
Pitch is the same as the x angle of an entity, and is equivalent to tilting forward/backwards. <br><br>
Yaw is the same as the y angle of an entity, and is equivalent to turning left/right. <br><br>
Roll is the same as the z angle of an entity, and is equivalent to tilting left/right. 
EndRem
Function ib3d_RotateEntity(entity:ib3d_Entity,pitch:Float,yaw:Float,roll:Float,globl:Int=False)
	If entity=Null Then Throw "Entity does not exist" 
	entity.Rotate(pitch,yaw,roll,globl)
EndFunction

Rem
bbdoc: Moves an entity relative to its current position and orientation
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
<li>x:Float - x amount entity will be moved by</li>
<li>y:Float - y amount entity will be moved by</li>
<li>z:Float - z amount entity will be moved by</li>
</ul>
What this means is that an entity will move in whatever direction it is facing. So for example if you have an game character is upright when first loaded into Blitz3D and it remains upright (i.e. turns left or right only), then moving it by a z amount will always see it move forward or backward, moving it by a y amount will always see it move up or down, and moving it by an x amount will always see it strafe. 
EndRem
Function ib3d_MoveEntity(entity:ib3d_Entity,x:Float,y:Float,z:Float)
	If entity=Null Then Throw "Entity does not exist" 
	entity.Move(x,y,z)
EndFunction

Rem
bbdoc: Turns an entity relative to its current orientation
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
<li>pitch:Float - angle in degress the entity will be pitched</li>
<li>yaw:Float - angle in degrees the entity will be yawed</li>
<li>roll:Float - angle in degrees the entity will be rolled</li>
</ul>
Pitch is the same as the x angle of an entity, and is equivalent to tilting forward/backwards. <br><br>
Yaw is the same as the y angle of an entity, and is equivalent to turning left/right. <br><br>
Roll is the same as the z angle of an entity, and is equivalent to tilting left/right. 
EndRem
Function ib3d_TurnEntity(entity:ib3d_Entity,pitch:Float,yaw:Float,roll:Float,globl:Int=False)
	If entity=Null Then Throw "Entity does not exist" 
	entity.Turn(pitch,yaw,roll,globl)
EndFunction

Rem
bbdoc: Sets the speed at which an entity moves and rotates.
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
<li>new_speed:Float - speed to alter the entity with.  Default is 1.0.</li>
</ul>
WIP - This setting affects Move and Turn commands.  Each move/turn is multiplied by the speed setting.<br><br>
This setting is independent of the engine speed and the animation speed.
EndRem
Function ib3d_EntitySetSpeed(entity:ib3d_Entity,new_speed:Float=1.0)
	If entity=Null Then Throw "Entity does not exist" 
	entity.setSpeed(new_speed)
EndFunction

Rem
bbdoc: Returns the current entity speed setting.
returns: The current speed setting.
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
</ul>
EndRem
Function ib3d_EntityGetSpeed:Float(entity:ib3d_Entity)
	If entity=Null Then Throw "Entity does not exist" 
	Return entity.getSpeed()
EndFunction

Rem
bbdoc: Points one entity at another
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
<li>target:ib3d_Entity - target entity</li>
<li>roll:Float - roll angle of entity.  Defaults to 0.0.</li>
</ul>
NOTE: Completely hosed....<br><br>
The optional roll parameter allows you to specify a roll angle as pointing an entity only sets pitch and yaw angles. <br><br>
If you wish for an entity to point at a certain position rather than another entity, simply create a pivot entity at your desired position, point the entity at this and then free the pivot. 
EndRem
Function ib3d_PointEntity(entity:ib3d_Entity,target:ib3d_Entity,roll:Float=0.0)
	If entity=Null Then Throw "Entity does not exist" 
	entity.Point(target)
EndFunction

Rem
bbdoc: Sets the color of an entity
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
<li>red:Float - red value of entity</li>
<li>green:Float - green value of entity</li>
<li>blue:Float - blue value of entity</li>
<li>MaterialIDX:Int - Entity material to set the color for. Defaults to 0.</li>
</ul>
The Red, Green and Blue values should be in the range 0-255 with 0 being darkest and 255 brightest. The default entity color is 255,255,255 (White).
EndRem
Function ib3d_EntityColor(entity:ib3d_Entity,red:Float,green:Float,blue:Float,MaterialIDX:Int=0)
	If entity=Null Then Throw "Entity does not exist" 
	entity.setColor(red,green,blue,MaterialIDX)
EndFunction

Rem
bbdoc: How much ambient light (a global light) is reflected by this entity
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
<li>red:Float - red value for ambient light</li>
<li>green:Float - green value for ambient light</li>
<li>blue:Float - blue value for ambient light</li>
<li>MaterialIDX:Int - Index of the material to set the ambient color for.  Defaults to 0.</li>
</ul>
The default is full white, meaning objects are completely globally illuminated. Reduce this if you want to see diffuse or specular light effects. 
EndRem
Function ib3d_EntityAmbientColor(entity:ib3d_Entity,red:Float,green:Float,blue:Float,MaterialIDX:Int=0)
	If entity=Null Then Throw "Entity does not exist" 
	entity.setAmbientColor(red,green,blue,MaterialIDX)
EndFunction

Rem
bbdoc: How much diffuse light coming from a light source is reflected by this entity.
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
<li>red:Float - red value for diffuse light</li>
<li>green:Float - green value for diffuse light</li>
<li>blue:Float - blue value for diffuse light</li>
<li>MaterialIDX:Int - Index of the material to set the diffuse color for.  Defaults to 0.</li>
</ul>
The default is full white. 
EndRem
Function ib3d_EntityDiffuseColor(entity:ib3d_Entity,red:Float,green:Float,blue:Float,MaterialIDX:Int=0)
	If entity=Null Then Throw "Entity does not exist" 
	entity.setDiffuseColor(red,green,blue,MaterialIDX)
EndFunction

Rem
bbdoc: How much specular light (highlights from a light) is reflected
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
<li>red:Float - red value for specular light</li>
<li>green:Float - green value for specular light</li>
<li>blue:Float - blue value for specular light</li>
<li>MaterialIDX:Int - Index of the material to set the specular color for.  Defaults to 0.</li>
</ul>
The default is to reflect white specular light. See ib3d_EntityShininess how to enable specular lights. 
EndRem
Function ib3d_EntitySpecularColor(entity:ib3d_Entity,red:Float,green:Float,blue:Float,MaterialIDX:Int=0)
	If entity=Null Then Throw "Entity does not exist" 
	entity.setSpecularColor(red,green,blue,MaterialIDX)
EndFunction

Rem
bbdoc: Light emitted by this entity
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
<li>red:Float - red value for emissive light</li>
<li>green:Float - green value for emissive light</li>
<li>blue:Float - blue value for emissive light</li>
<li>MaterialIDX:Int - Index of the material to set the emissive color for.  Defaults to 0.</li>
</ul>
Default is to emitt no light.
EndRem
Function ib3d_EntityEmissiveColor(entity:ib3d_Entity,red:Float,green:Float,blue:Float,MaterialIDX:Int=0)
	If entity=Null Then Throw "Entity does not exist" 
	entity.setEmissiveColor(red,green,blue,MaterialIDX)
EndFunction

Rem
bbdoc: Sets miscellaneous effects for an entity
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
<li>E_MATERIAL_FLAG:Int - the effect to set</li>
<li>bNewVal:Int - the new value for the effect</li>
<li>MaterialIDX:Int - the material to set the effect on.  Default is -1 for all materials.</li>
</ul>
Material flags:<br>
EMF_WIREFRAME: Draw as wireframe Or filled triangles? Default: False<br>
EMF_POINTCLOUD: Draw as point cloud Or filled triangles? Default: False<br>
EMF_GOURAUD_SHADING: Flat Or Gouraud shading? Default: True<br>
EMF_LIGHTING: Will this material be lighted? Default: True<br>
EMF_ZBUFFER: Is the ZBuffer enabled? Default: True<br>
EMF_ZWRITE_ENABLE: May be written To the zbuffer Or is it readonly. Default: True. NOTE: This flag is ignored, If the material Type is a transparent type.<br>
EMF_BACK_FACE_CULLING: Is backfaceculling enabled? Default: True<br>
EMF_BILINEAR_FILTER: Is bilinear filtering enabled? Default: True<br>
EMF_TRILINEAR_FILTER: Is trilinear filtering enabled? Default: False.  NOTE: If the trilinear filter flag is enabled, the bilinear filtering flag is ignored.<br>
EMF_ANISOTROPIC_FILTER: Is anisotropic filtering? Default: False.  NOTE: In Irrlicht you can use anisotropic texture filtering in conjunction with bilinear Or trilinear texture filtering To improve rendering results. Primitives will look less blurry with this flag switched on.<br>
EMF_FOG_ENABLE: Is fog enabled? Default: False<br>
EMF_NORMALIZE_NORMALS: Normalizes normals.You can enable this if you need to scale a dynamic lighted model. Usually, its normals will get scaled too then and it will get darker. If you enable the EMF_NORMALIZE_NORMALS flag, the normals will be normalized again, and the model will look as bright as it should.<br>
EndRem
Function ib3d_EntityFX(entity:ib3d_Entity,E_MATERIAL_FLAG:Int,bNewVal:Int,MaterialIDX:Int=-1)
	If entity=Null Then Throw "Entity does not exist" 
	entity.setFX(E_MATERIAL_FLAG,bNewVal,MaterialIDX)
EndFunction

Rem
bbdoc: Value affecting the size of specular highlights
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
<li>shininess:Float - Size of the specular highlight.  Defaults to 0.</li>
<li>MaterialIDX:Int - the material to set the shininess on.  Default is 0.</li>
</ul>
If set to 0, no specular highlights are being used. To activate, simply set the shininess of a material to a value other than 0.
EndRem
Function ib3d_EntityShininess(entity:ib3d_Entity,shininess:Float=0.0,MaterialIDX:Int=0)
	If entity=Null Then Throw "Entity does not exist" 
	entity.setShininess(shininess,MaterialIDX)
EndFunction

' TODO: new params
Rem
bbdoc: Sets the blending mode of an entity.
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
<li>E_MATERIAL_TYPE:Int - the blending mode.  Defaults to EMT_SOLID.</li>
<li>MaterialIDX:Int - the material to set the blend on.  Default is -1 for all materials.</li>
</ul>
Blend Modes:<br>
<p>
EMT_SOLID<br>
Standard solid material. Only first texture is used= which is supposed To be the diffuse material.
</p>
<p>
EMT_SOLID_2_LAYER<br>
Solid material with 2 texture layers. The second is blended onto the first using the alpha value of the vertex colors. This material is currently Not implemented in OpenGL but it works with DirectX.
</p>
<p>
EMT_LIGHTMAP<br>
Material Type with standard lightmap technique: There should be 2 textures: The first texture layer is a diffuse map the second is a light map. Vertex light is ignored.
</p>
<p>
EMT_LIGHTMAP_ADD<br>	
Material Type with lightmap technique like EMT_LIGHTMAP but lightmap And diffuse texture are Not modulated= but added instead.
</p>
<p>
EMT_LIGHTMAP_M2<br>	
Material Type with standard lightmap technique: There should be 2 textures: The first texture layer is a diffuse map the second is a light map. Vertex light is ignored. The texture colors are effectively multiplyied by 2 For brightening. like known in DirectX as D3DTOP_MODULATE2X.
</p>
<p>
EMT_LIGHTMAP_M4<br>	
Material Type with standard lightmap technique: There should be 2 textures: The first texture layer is a diffuse map the second is a light map. Vertex light is ignored. The texture colors are effectively multiplyied by 4 For brightening. like known in DirectX as D3DTOP_MODULATE4X.
</p>
<p>
EMT_LIGHTMAP_LIGHTING<br>
Like EMT_LIGHTMAP but also supports dynamic lighting.
</p>
<p>
EMT_LIGHTMAP_LIGHTING_M2<br>
Like EMT_LIGHTMAP_M2 but also supports dynamic lighting.
</p>
<p>
EMT_LIGHTMAP_LIGHTING_M4<br>
Like EMT_LIGHTMAP_4 but also supports dynamic lighting.
</p>
<p>
EMT_DETAIL_MAP<br>
Detail mapped material. The first texture is diffuse color map, the second is added to this and usually displayed with a bigger scale value so that it adds more detail. The detail map is added to the diffuse map using ADD_SIGNED, so that it is possible to add and substract color from the diffuse map. For example a value of (127,127,127) will not change the appearance of the diffuse map at all. Often used for terrain rendering. 
</p>
<p>
EMT_SPHERE_MAP<br>	
Makes the material look like it was reflection the environment around it. To make this possible= a texture called 'sphere map' is used= which must be set as Texture1.
</p>
<p>
EMT_REFLECTION_2_LAYER<br>
A reflecting material with an optional additional non reflecting texture layer. The reflection map should be set as Texture 1. Please note that this material Type is currently Not 100% implemented in OpenGL. It works in DirectX8 And 9.
</p>
<p>
EMT_TRANSPARENT_ADD_COLOR<br>
A transparent material. Only the first texture is used. The New color is calculated by simply adding the source color And the dest color. This means If For example a billboard using a texture with black background And a red circle on it is drawed with this material the result is that only the red circle will be drawn a little bit transparent And everything which was black is 100% transparent And Not visible. This material Type is useful For e.g. particle effects.
</p>
<p>
EMT_TRANSPARENT_ALPHA_CHANNEL<br>
Makes the material transparent based on the texture alpha channel. Only first texture is used. If you are using this material with small textures And 3d Object= it is a good idea To load the texture in 32 bit mode (video::IVideoDriver::setTextureCreationFlag()).
</p>
<p>
EMT_TRANSPARENT_ALPHA_CHANNEL_REF<br>
Makes the material transparent based on the texture alpha channel. If the alpha channel value is greater than 127, a pixel is written To the target, otherwise not. This material does Not use alpha blending And is a lot faster than EMT_TRANSPARENT_ALPHA_CHANNEL. It is ideal For drawing stuff like leafes of plants, because the borders are Not blurry but sharp. Only first texture is used. If you are using this material with small textures And 3d Object, it is a good idea To load the texture in 32 bit mode (video::IVideoDriver::setTextureCreationFlag()).
</p>
<p>
EMT_TRANSPARENT_VERTEX_ALPHA<br>
Makes the material transparent based on the vertex alpha value.
</p>
<p>
EMT_TRANSPARENT_REFLECTION_2_LAYER<br>
A transparent reflecting material with an optional additional non reflecting texture layer. The reflection map should be set as Texture 1. The transparency depends on the alpha value in the vertex colors. A texture which will Not reflect can be set als Texture 2. Please note that this material Type is currently Not 100% implemented in OpenGL. It works in DirectX8.
</p>
<p>
EMT_NORMAL_MAP_SOLID<br>
A solid normal map renderer. First texture is the color map the second should be the normal map. Note that you should use this material only when drawing geometry consisting of vertices of Type S3DVertexTangents (EVT_TANGENTS). You can convert any mesh into this format using IMeshManipulator::createMeshWithTangents() (See SpecialFX2 Tutorial). This shader runs on vertex shader 1.1 And pixel shader 1.1 capable hardware And falls back on a fixed Function lighted material If this hardware is Not available.  Only two lights are supported by this shader= If there are more= the nearest two are chosen. Currently= this shader is only implemented For the D3D8 And D3D9 renderers.
</p>
<p>
EMT_NORMAL_MAP_TRANSPARENT_ADD_COLOR<br>
A transparent normal map renderer. First texture is the color map the second should be the normal map. Note that you should use this material only when drawing geometry consisting of vertices of Type S3DVertexTangents (EVT_TANGENTS). You can convert any mesh into this format using IMeshManipulator::createMeshWithTangents() (See SpecialFX2 Tutorial). This shader runs on vertex shader 1.1 And pixel shader 1.1 capable hardware And falls back on a fixed Function lighted material If this hardware is Not available. Only two lights are supported by this shader= If there are more= the nearest two are chosen. Currently= this shader is only implemented For the D3D8 And D3D9 renderers.
</p>
<p>
EMT_NORMAL_MAP_TRANSPARENT_VERTEX_ALPHA<br>
A transparent (based on the vertex alpha value) normal map renderer. First texture is the color map the second should be the normal map. Note that you should use this material only when drawing geometry consisting of vertices of Type S3DVertexTangents (EVT_TANGENTS). You can convert any mesh into this format using IMeshManipulator::createMeshWithTangents() (See SpecialFX2 Tutorial). This shader runs on vertex shader 1.1 And pixel shader 1.1 capable hardware And falls back on a fixed Function lighted material If this hardware is Not available. Only two lights are supported by this shader= If there are more= the nearest two are chosen. Currently= this shader is only implemented For the D3D8 And D3D9 renderers.
</p>
<p>
EMT_PARALLAX_MAP_SOLID<br>
Just like EMT_NORMAL_MAP_SOLID, but uses parallax mapping too, which looks a lot more realistic. This only works when the hardware supports at least vertex shader 1.1 And pixel shader 1.4. First texture is the color map, the  second should be the normal map. The normal map texture should contain the height value in the alpha component. The IVideoDriver::makeNormalMapTexture() Method writes this value automaticly when creating normal maps from a heightmap when using a 32 bit texture. The height scale of the material (affecting the bumpiness) is being controlled by the SMaterial::MaterialTypeParam member.  If set To zero, the Default value (0.02f) will be applied. Otherwise the value set in SMaterial::MaterialTypeParam is taken. This value depends on with which scale the texture is mapped on the material. Too high Or low values of MaterialTypeParam can result in strange artifacts. 
</p>
<p>
EMT_PARALLAX_MAP_TRANSPARENT_ADD_COLOR<br>
A material just like EMT_PARALLAX_MAP_SOLID, but it is transparent, using EMT_TRANSPARENT_ADD_COLOR as base material.
</p>
<p>
EMT_PARALLAX_MAP_TRANSPARENT_VERTEX_ALPHA<br>
A material just like EMT_PARALLAX_MAP_SOLID, but it is transparent, using EMT_TRANSPARENT_VERTEX_ALPHA as base material.
</p>
EndRem
Function ib3d_EntityBlend(entity:ib3d_Entity,E_MATERIAL_TYPE:Int=EMT_SOLID,MaterialIDX:Int=-1)
	If entity=Null Then Throw "Entity does not exist" 
	entity.setBlend(E_MATERIAL_TYPE,MaterialIDX)
EndFunction

Rem
bbdoc: Returns the distance between the passed entities
returns: The distance
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
<li>other:ib3d_Entity - other entity to check distance from</li>
</ul>
EndRem
Function ib3d_EntityDistance:Double(entity:ib3d_Entity,other:ib3d_Entity)
	If entity=Null Then Throw "Entity does not exist" 
	If other=Null Then Throw "Distance entity does not exist" 
	Return entity.getDistanceFrom(other)
EndFunction

Rem 

EntityPick - same as entityvisible, but take nearest one.  
- need to calculate direction facing.  see cameraupdate.
LinePick - getSceneNodeFromRayBB()
EntityVisible - getSceneNodeFromRayBB(), check the pickmode.  need to investigate the bits setting.	
- in theory, can pick based on center and points of box.  if any return the other entity then yes.  also 
- use the distance to create the ray.
TFormPoint - the goal here is to convert a point from local to global and then to local of a differnt type
TFormVector	- same as point
TFormNormal	- same as point
TFormedX - returns X of last call
TFormedY - returns Y of last call
TFormedZ - returns Z of last call
EntityAnimating	
EntityAnimTime

Function ib3d_EntityBox(entity:ISceneNode,x:Float,y:Float,z:Float,width:Float,height:Float,depth:Float)
	If (entity<>Null And entity.isValid())
		entity.setParent(parent)
	Else
		Throw "Entity does not exist"
	EndIf
EndFunction
EndRem


Rem

- EntityMovement
AlignToVector 
PositionEntity 
* note - need to figure out global math.  need to translate local position  global position based on parent position.
(done) RotateEntity - ISceneNode.setRotation()
(done) TurnEntity - ISceneNode.setRotation(),ISceneNode.getRotation().Plus(vector3df)

TranslateEntity 
PointEntity 
CopyEntity 

EntityAlpha - transparency in this form is not yet supported in Irrlicht
EntityAutoFade - once EntityAlpha is working, can do with by calculating distance from camera and setting level of trans automatically	
PaintEntity	
EntityOrder 


EndRem

