' ------------------------------------------------------------------------------------
' irr::core::triangle3df wrappers (done)
' ------------------------------------------------------------------------------------

Public

Type Triangle3df Extends IrrBase Final

	Method closestPointOnTriangle:Vector3df(pVect:Vector3df)
		Return Vector3df.createFromHandle(IrrCore_Triangle3df_closestPointOnTriangle(handle,pVect.handle))
	EndMethod

	Method isFrontFacing:Int(pLookDirection:Vector3df)
		Return IrrCore_Triangle3df_isFrontFacing(handle,pLookDirection.handle)
	EndMethod

	Method isTotalInsideBox:Int(pBox:AABbox3df)
		Return IrrCore_Triangle3df_isTotalInsideBox(handle,pBox.handle)
	EndMethod
	
	Method isPointInside:Int(pVect:Vector3df)
		Return IrrCore_Triangle3df_isPointInside(handle,pVect.handle)
	EndMethod

	Method isPointInsideFast:Int(pVect:Vector3df)
		Return IrrCore_Triangle3df_isPointInsideFast(handle,pVect.handle)
	EndMethod
	
	Method EqEq:Int(pTriOther:Triangle3df)
		Return IrrCore_Triangle3df_EqEq(handle,pTriOther.handle)
	EndMethod

	Method NotEq:Int(pTriOther:Triangle3df)
		Return IrrCore_Triangle3df_NotEq(handle,pTriOther.handle)
	EndMethod

	'--------------------------------------------
	' get routines
	'--------------------------------------------

	Method getIntersectionWithLimitedLine:Int(pLine:Line3df,pOutIntersection:Vector3df)
		Return IrrCore_Triangle3df_getIntersectionWithLimitedLine(handle,pLine.handle,pOutIntersection.handle)
	EndMethod

	Method getIntersectionWithLine:Int(pLinePoint:Vector3df,pLineVect:Vector3df,pOutIntersection:Vector3df)
		Return IrrCore_Triangle3df_getIntersectionWithLine(handle,pLinePoint.handle,pLineVect.handle,pOutIntersection.handle)
	EndMethod

	Method getIntersectionOfPlaneWithLine:Int(pLinePoint:Vector3df,pLineVect:Vector3df,pOutIntersection:Vector3df)
		Return IrrCore_Triangle3df_getIntersectionOfPlaneWithLine(handle,pLinePoint.handle,pLineVect.handle,pOutIntersection.handle)
	EndMethod

	Method getNormal:Vector3df()
		Return Vector3df.createFromHandle(IrrCore_Triangle3df_getNormal(handle))
	EndMethod
	
	Method getPlane:Plane3df()
		Return Plane3df.createFromHandle(IrrCore_Triangle3df_getPlane(handle))
	EndMethod
	
	Method getPointA:Vector3df()
		' set to not delete
		Return Vector3df.createFromHandle(IrrCore_Triangle3df_getPointA(handle),False)
	EndMethod
	
	Method getPointB:Vector3df()
		' set to not delete
		Return Vector3df.createFromHandle(IrrCore_Triangle3df_getPointB(handle),False)
	EndMethod

	Method getPointC:Vector3df()
		' set to not delete
		Return Vector3df.createFromHandle(IrrCore_Triangle3df_getPointC(handle),False)
	EndMethod

	Method getPoint:Vector3df(index:Int)
		If index = 0 Then Return getPointA()
		If index = 1 Then Return getPointB()
		If index = 2 Then Return getPointC()
		Return Null
	EndMethod

	'--------------------------------------------
	' set routines
	'--------------------------------------------

	Method setFromVect(pVect1:Vector3df,pVect2:Vector3df,pVect3:Vector3df)
		IrrCore_Triangle3df_setFromVect(handle,pVect1.handle,pVect2.handle,pVect3.handle)
	EndMethod
	
	Method setFromVals(x1:Float=0,y1:Float=0,z1:Float=0,x2:Float=0,y2:Float=0,z2:Float=0,x3:Float=0,y3:Float=0,z3:Float=0)
		IrrCore_Triangle3df_setFromVals(handle,x1,y1,z1,x2,y2,z2,x3,y3,z3)
	EndMethod

	Method setPointA(pVect:Vector3df)
		IrrCore_Triangle3df_setPointA(handle,pVect.handle)
	EndMethod

	Method setPointB(pVect:Vector3df)
		IrrCore_Triangle3df_setPointB(handle,pVect.handle)
	EndMethod

	Method setPointC(pVect:Vector3df)
		IrrCore_Triangle3df_setPointC(handle,pVect.handle)
	EndMethod

	Method getArea:Float()
		Return IrrCore_Triangle3df_getArea(handle)
	EndMethod

	'--------------------------------------------
	' create/delete routines
	'--------------------------------------------

	Function create:Triangle3df()
		Return createFromHandle(IrrCore_Triangle3df_new())
	EndFunction

	Function createFromVals:Triangle3df(x1:Float=0,y1:Float=0,z1:Float=0,x2:Float=0,y2:Float=0,z2:Float=0,x3:Float=0,y3:Float=0,z3:Float=0)
		Return createFromHandle(IrrCore_Triangle3df_newFromVals(x1,y1,z1,x2,y2,z2,x3,y3,z3))
	EndFunction
	
	Function createFromVect:Triangle3df(pVect1:Vector3df,pVect2:Vector3df,pVect3:Vector3df)
		Return createFromHandle(IrrCore_Triangle3df_newFromVect(pVect1.handle,pVect2.handle,pVect3.handle))
	EndFunction
	
	Function createFromHandle:Triangle3df(pTri:Int,bMustDel:Int=True)
		If Not pTri Then Return Null
		Local oRetVal:Triangle3df=New Triangle3df
		oRetVal.attach(pTri,bMustDel)
		Return oRetVal
	EndFunction

	Method Delete()
		If (mustdelete) Then IrrCore_Triangle3df_del(handle)
	EndMethod	
EndType

