' ------------------------------------------------------------------------------------
' irr::IrrlichtDevice wrappers (done)
' ------------------------------------------------------------------------------------
Public

Type IrrlichtDevice Extends Tirr_IUnknown Final

	Function create:IrrlichtDevice(driver:Int,size:Dimension2di,bits:Int=16,bFullscreen:Int=False,bStencilbuffer:Int=False,bVsync:Int=False,evt:IEventReceiver=Null)
		If evt=Null Then evt=New IEventReceiver ' set handle to 0
		Return createFromHandle(Irr_createDevice(driver,size.handle,bits,bFullscreen,bStencilbuffer,bVsync,evt.handle))
	EndFunction

	Function createFromParams:IrrlichtDevice(params:SIrrlichtCreationParameters)
		Return createFromHandle(Irr_createDeviceEx(params.handle))
	EndFunction

	Function createFromHandle:IrrlichtDevice(pHandle:Int,bMustDel:Int=True)
		If Not pHandle Then Return Null
		Local oRetVal:IrrlichtDevice=New IrrlichtDevice
		oRetVal.attach(pHandle,bMustDel)
		If oRetVal.getVersion()<>CompatibleDLLVersion Then .. 
			RuntimeError "The Irrlicht library version "+oRetVal.getVersion()+ ..
			" does not match the mod version "+CompatibleDLLVersion
		Return oRetVal
	EndFunction

	Method drop:Int() Final
		If mustdelete Then mustdelete=False		
		Return Irr_IrrlichtDevice_drop(handle)
	EndMethod
	
	Method getDebugName:String() Final
		Return Irr_IrrlichtDevice_getDebugName(handle)
	EndMethod
	
	Method grab() Final
		Irr_IrrlichtDevice_grab(handle)
	EndMethod
	
	Method Delete()
		If (mustdelete) Then drop()
	EndMethod

	Method setResizeAble(bResize:Int)
		Irr_IrrlichtDevice_setResizeAble(handle,bResize) 
	EndMethod
	
	Method getEventReceiver:IEventReceiver()
		Return IEventReceiver.createFromHandle(IEventReceiver.generate,Irr_IrrlichtDevice_getEventReceiver(handle),False)
	EndMethod

	Method setEventReceiver(rec:IEventReceiver)
		Irr_IrrlichtDevice_setEventReceiver(handle,rec.handle)
	EndMethod
	
	Method isWindowActive:Int()
		Return Irr_IrrlichtDevice_isWindowActive(handle)
	EndMethod
	
	Method getVideoModeList:IVideoModeList()
		Return IVideoModeList.createFromHandle(Irr_IrrlichtDevice_getVideoModeList(handle))
	EndMethod
	
	Method getVersion:String()
		Return String.FromCString(Irr_IrrlichtDevice_getVersion(handle))
	EndMethod
	
	Method getTimer:ITimer()
		Return ITimer.createFromHandle(Irr_IrrlichtDevice_getTimer(handle))
	EndMethod
	
	Method getOSOperator:IOSOperator()
		Return IOSOperator.createFromHandle(Irr_IrrlichtDevice_getOSOperator(handle))
	EndMethod
	
	Method getLogger:ILogger()
		Return ILogger.createFromHandle(Irr_IrrlichtDevice_getLogger(handle))
	EndMethod
	
	Method getFileSystem:IFileSystem()
		Return IFileSystem.createFromHandle(Irr_IrrlichtDevice_getFileSystem(handle))
	EndMethod 
	
	Method closeDevice()
		Irr_IrrlichtDevice_closeDevice(handle)
	EndMethod
	
	Method getCursorControl:ICursorControl()
		Return ICursorControl.createFromHandle(Irr_IrrlichtDevice_getCursorControl(handle))
	EndMethod
	
	Method setWindowCaption(text:String)
		Irr_IrrlichtDevice_setWindowCaption(handle,text)
	EndMethod

	Method setWindowCaptionW(text:String)
		Irr_IrrlichtDevice_setWindowCaptionW(handle,text)
	EndMethod
	
	Method run:Int()
		Return Irr_IrrlichtDevice_run(handle)
	EndMethod
	
	Method getVideoDriver:IVideoDriver()
		Return IVideoDriver.createFromHandle(Irr_IrrlichtDevice_getVideoDriver(handle))
	EndMethod
	
	Method getSceneManager:ISceneManager()
		Return ISceneManager.createFromHandle(Irr_IrrlichtDevice_getSceneManager(handle))
	EndMethod

	Method getGUIEnvironment:IGUIEnvironment()	
		Return IGUIEnvironment.createFromHandle(Irr_IrrlichtDevice_getGUIEnvironment(handle))
	EndMethod
	
	Method postEventFromUser(event:SEvent)
		Irr_IrrlichtDevice_postEventFromUser(handle,event.handle)
	EndMethod
	
	Method yield()
		Irr_IrrlichtDevice_yield(handle)
	EndMethod
	
	Method sleep(timeMs:Int,bPauseTimer:Int=False)
		Irr_IrrlichtDevice_sleep(handle,timeMs,bPauseTimer)
	EndMethod

	Method setInputReceivingSceneManager(sceneManager:ISceneManager)
		Irr_IrrlichtDevice_setInputReceivingSceneManager(handle,sceneManager.handle)
	EndMethod
	
EndType
