' ------------------------------------------------------------------------------------
' irr::scene::IMeshLoader wrappers
' ------------------------------------------------------------------------------------
Public

Type IMeshLoader Extends Tirr_IUnknown

	' --------------------------------------------------------------------------------
	' override these in extended type to implement new functionality
	' --------------------------------------------------------------------------------

	Method createMesh:IAnimatedMesh(file:IReadFile) Abstract

	Method isALoadableFileExtension:Int(filename:String) Abstract
	
	' --------------------------------------------------------------------------------
	' base methods
	' --------------------------------------------------------------------------------
	
	Method drop:Int() Final
		If mustdelete Then mustdelete=False		
		Return IrrScene_IMeshLoader_drop(handle)
	EndMethod
	
	Method getDebugName:String() Final
		Return IrrScene_IMeshLoader_getDebugName(handle)
	EndMethod
	
	Method grab() Final
		IrrScene_IMeshLoader_grab(handle)
	EndMethod

	Method Delete()
		If (mustdelete) Then drop()
	EndMethod
	' --------------------------------------------------------------------------------
	' creation routines
	' --------------------------------------------------------------------------------

	' this should only be called on an extended type
	Function create:IMeshLoader(pGenFunc:Byte Ptr) Final
		Return createFromHandle(IrrScene_BMXIMeshLoader_new(),True,pGenFunc)
	EndFunction

	Function createFromHandle:IMeshLoader(pHandle:Int,bMustDel:Int,pGenFunc:Byte Ptr=Null) Final
		If Not pHandle Then Return Null
		Local oRetVal:IMeshLoader=Null
		
		If pGenFunc
			' use the funtion pointer to generate
			Local generate:IMeshLoader()=pGenFunc
		
			oRetVal=generate()
		Else
			oRetVal=New IMeshLoaderDefault
		EndIf

		' turn binding on if this is the first and there was a gen func passed in
		oRetVal.bIsBound=bMustDel And pGenFunc
		
		oRetVal.attach(pHandle,bMustDel)
		
		Return oRetVal
	EndFunction

	' --------------------------------------------------------------------------------
	' bindings
	' --------------------------------------------------------------------------------

	Method bind(rec:Int) Final
		Super.bind(rec)
		IrrScene_BMXIMeshLoader_bindCreateMesh( ..
			translate(rec).handle, ..
			OnCreateMeshBinding)
		IrrScene_BMXIMeshLoader_bindIsALoadableFileExtension( ..
			translate(rec).handle, ..
			OnIsALoadableFileExtensionBinding)
	EndMethod

	Function OnCreateMeshBinding:Int(binding:Int,pIReadFile:Int) Final
		' get the object whose node is firing
		Local tempobj:IMeshLoader=IMeshLoader(translate(binding))
		
		If tempobj<>Null
	  		' get the reader
  			Local tempreader:IReadFile=IReadFile.createFromHandle(pIReadFile,False)
  			' fire the event
			Local retval:IAnimatedMesh=tempobj.createMesh(tempreader)
	  		Return retval.handle
		Else
			Throw "Unable to find bound object"
		EndIf
	EndFunction

	Function OnIsALoadableFileExtensionBinding:Int(binding:Int,filename:Byte Ptr) Final
		' get the object whose node is firing
		Local tempobj:IMeshLoader=IMeshLoader(translate(binding))
		
		If tempobj<>Null
  			' fire the event
	  		Return tempobj.isALoadableFileExtension(String.FromCString(filename))
		Else
			Throw "Unable to find bound object"
		EndIf	
	EndFunction

	Method getBinderHandle:Int()
		Return IrrScene_BMXIMeshLoader_getBinder(handle)
	EndMethod
EndType

' ------------------------------------------------------------------------
' default implementation of IMeshLoader
' ------------------------------------------------------------------------
Type IMeshLoaderDefault Extends IMeshLoader

	' --------------------------------------------------------------------------------
	' override these in extended type to implement new functionality
	' --------------------------------------------------------------------------------
	Method createMesh:IAnimatedMesh(file:IReadFile)
		Return SAnimatedMesh.createFromHandle(IrrScene_IMeshLoader_createMesh(handle,file.handle),True)
	EndMethod
	
	Method isALoadableFileExtension:Int(filename:String)
		Return IrrScene_IMeshLoader_isALoadableFileExtension(handle,filename)
	EndMethod
EndType
