' ------------------------------------------------------------------------------------
' irr::video::IImageWriter wrapper
' ------------------------------------------------------------------------------------
Public

Type IImageWriter Extends Tirr_IUnknown
	
	' --------------------------------------------------------------------------------
	' override these in extended type to implement new functionality
	' --------------------------------------------------------------------------------
	
	Method isAWriteableFileExtension:Int(fileName:String)
		Return False
	EndMethod
	
	Method writeImage:Int(writer:IWriteFile,image:IImage,param:Int=0)
		Return False
	EndMethod
	
	Method drop:Int() Final
		If mustdelete Then mustdelete=False		
		Return IrrVideo_IImageWriter_drop(handle)
	EndMethod
	
	Method getDebugName:String() Final
		Return IrrVideo_IImageWriter_getDebugName(handle)
	EndMethod
	
	Method grab() Final
		IrrVideo_IImageWriter_grab(handle)
	EndMethod

	Method Delete()
		If (mustdelete) Then drop()
	EndMethod
	
	' --------------------------------------------------------------------------------
	' creation routines
	' --------------------------------------------------------------------------------

	' this should only be called on an extended type
	Function create:IImageWriter(pGenFunc:Byte Ptr) Final
		Return createFromHandle(IrrVideo_BMXIImageWriter_new(),True,pGenFunc)
	EndFunction

	Function createFromHandle:IImageWriter(pHandle:Int,bMustDel:Int,pGenFunc:Byte Ptr=Null) Final
		If Not pHandle Then Return Null
		Local oRetVal:IImageWriter=Null
		
		If pGenFunc
			' use the funtion pointer to generate
			Local generate:IImageWriter()=pGenFunc
		
			oRetVal=generate()
		Else
			oRetVal=New IImageWriter
		EndIf

		' turn binding on if this is the first and there was a gen func passed in
		oRetVal.bIsBound=bMustDel And pGenFunc
		
		oRetVal.attach(pHandle,bMustDel)
		
		Return oRetVal
	EndFunction

	' --------------------------------------------------------------------------------
	' bindings
	' --------------------------------------------------------------------------------

	Method bind(rec:Int) Final
		Super.bind(rec)
		IrrVideo_BMXIImageWriter_bindWriteImage( ..
			translate(rec).handle, ..
			OnWriteImage)
		IrrVideo_BMXIImageWriter_bindIsAWriteableFileExtension( ..
			translate(rec).handle, ..
			OnIsAWriteableFileExtension)
	EndMethod

	Method getBinderHandle:Int()
		Return IrrVideo_BMXIImageWriter_getBinder(handle)
	EndMethod

	Function OnWriteImage:Int(binding:Int,writer:Int,image:Int,param:Int=0)
		' get the object whose node is firing
		Local tempobj:IImageWriter=IImageWriter(translate(binding))
		
		If tempobj<>Null
  			' fire the event
	  		Return tempobj.writeImage(IWriteFile.createFromHandle(writer,False),IImage.createFromHandle(image,False),param)
		Else
			Throw "Unable to find bound object"
		EndIf	
	EndFunction

	Function OnIsAWriteableFileExtension:Int(binding:Int,filename:Byte Ptr)
		' get the object whose node is firing
		Local tempobj:IImageWriter=IImageWriter(translate(binding))
		
		If tempobj<>Null
  			' fire the event
	  		Return tempobj.isAWriteableFileExtension(String.FromCString(filename))
		Else
			Throw "Unable to find bound object"
		EndIf	
	EndFunction
	
EndType

' ------------------------------------------------------------------------
' default implementation of IImageWriter
' ------------------------------------------------------------------------
Type IImageWriterDefault Extends IImageWriter	
	Method isAWriteableFileExtension:Int(fileName:String)
		Return IrrVideo_IImageWriter_isAWriteableFileExtension(handle,fileName)
	EndMethod
	
	Method writeImage:Int(writer:IWriteFile,image:IImage,param:Int=0)
		Return IrrVideo_IImageWriter_writeImage(handle,writer.handle,image.handle,param)
	EndMethod
EndType