' ------------------------------------------------------------------------------------
' irr::video::IVideoDriver wrappers
' use irr:IrrlichtDevice.getVideoDriver() To get a pointer For this
' ------------------------------------------------------------------------------------

Public

Type IVideoDriver Extends Tirr_IUnknown Final

	Function createFromHandle:IVideoDriver(pHandle:Int,bMustDel:Int=False)
		If Not pHandle Then Return Null
		Local oRetVal:IVideoDriver=New IVideoDriver
		oRetVal.attach(pHandle,bMustDel)
		Return oRetVal
	EndFunction

	Method beginScene:Int(bBackBuffer:Int,bZBuffer:Int,color:SColor=Null)
		If color=Null Then color=New SColor ' default handle to 0
		Return IrrVideo_IVideoDriver_beginScene(handle,bBackBuffer,bZBuffer,color.handle)
	EndMethod
	
	Method endScene:Int(windowId:Int=0,sourceRect:Recti=Null)
		If Not sourceRect Then sourceRect=New Recti ' default handle to 0
		Return IrrVideo_IVideoDriver_endScene(handle,windowId,sourceRect.handle)
	EndMethod
	
	Method queryFeature:Int(E_VIDEO_DRIVER_FEATURE:Int)
		Return IrrVideo_IVideoDriver_queryFeature(handle,E_VIDEO_DRIVER_FEATURE)
	EndMethod
	
	' do Not Delete Ptr
	Method getGPUProgrammingServices:IGPUProgrammingServices()
		Return IGPUProgrammingServices.createFromHandle(IrrVideo_IVideoDriver_getGPUProgrammingServices(handle),False)
	EndMethod

	Method SetTransform(E_TRANSFORMATION_STATE:Int,matrix:Matrix4)
		IrrVideo_IVideoDriver_setTransform(handle,E_TRANSFORMATION_STATE,matrix.handle)
	EndMethod
	
	Method getTransform:Matrix4(E_TRANSFORMATION_STATE:Int)
		Return Matrix4.createFromHandle(IrrVideo_IVideoDriver_getTransform(handle,E_TRANSFORMATION_STATE))
	EndMethod
	
	Method setMaterial(material:SMaterial)
		IrrVideo_IVideoDriver_setMaterial(handle,material.handle)
	EndMethod
	
	Method getTexture:ITexture(filename:String)
		Return ITexture.createFromHandle(IrrVideo_IVideoDriver_getTexture(handle,filename))
	EndMethod	

	Method getTextureFromStream:ITexture(read:IReadFile)
		Return ITexture.createFromHandle(IrrVideo_IVideoDriver_getTextureFromStream(handle,read.handle))
	EndMethod
	
	Method addTexture:ITexture(size:Dimension2di,name:String,ECOLOR_FORMAT:Int=ECF_A8R8G8B8)
		Return ITexture.createFromHandle(IrrVideo_IVideoDriver_addTexture(handle,size.handle,name,ECOLOR_FORMAT))
	EndMethod
	
	Method addTextureFromImage:ITexture(name:String,img:IImage)
		Return ITexture.createFromHandle(IrrVideo_IVideoDriver_addTextureFromImage(handle,name,img.handle))
	EndMethod
	
	Method removeTexture(texture:ITexture)
		IrrVideo_IVideoDriver_removeTexture(handle,texture.handle)
	EndMethod
	
	Method removeAllTextures()
		IrrVideo_IVideoDriver_removeAllTextures(handle)
	EndMethod
	
	Method makeColorKeyTexture(texture:ITexture,color:SColor)
		IrrVideo_IVideoDriver_makeColorKeyTexture(handle,texture.handle,color.handle)
	EndMethod
	
	Method makeColorKeyTextureFromPixel(texture:ITexture,pos:Position2di)
		IrrVideo_IVideoDriver_makeColorKeyTextureFromPixel(handle,texture.handle,pos.handle)
	EndMethod

	Method makeNormalMapTexture(texture:ITexture,amplitude:Float=1.0)	
		IrrVideo_IVideoDriver_makeNormalMapTexture(handle,texture.handle,amplitude)
	EndMethod 
	
	Method setRenderTarget(texture:ITexture=Null,bClearBackBuffer:Int=True,bClearZBuffer:Int=True,color:SColor=Null)
		If color=Null Then color=New SColor ' handle will be 0 tripping the default
		If texture=Null Then texture=New ITexture ' default handle to 0
		IrrVideo_IVideoDriver_setRenderTarget(handle,texture.handle,bClearBackBuffer,bClearZBuffer,color.handle)
	EndMethod

	Method SetViewport(area:Recti)
		IrrVideo_IVideoDriver_setViewPort(handle,area.handle)
	EndMethod 
	
	Method GetViewport:Recti()
		Return Recti.createFromHandle(IrrVideo_IVideoDriver_getViewPort(handle))
	EndMethod

	Method drawVertexPrimitiveList(vertices:Byte Ptr,vertexCount:Int,indexList:Short Ptr,triangleCount:Int,E_VERTEX_TYPE:Int,E_PRIMITIVE_TYPE:Int)
		IrrVideo_IVideoDriver_drawVertexPrimitiveList(handle,vertices,vertexCount,indexList,triangleCount,E_VERTEX_TYPE,E_PRIMITIVE_TYPE)
	EndMethod
	
	Method draw3DLine(start:Vector3df,pend:Vector3df,color:SColor=Null) ' def color=255,255,255,255
		If color=Null Then color=New SColor ' handle will be 0 tripping the default
		IrrVideo_IVideoDriver_draw3DLine(handle,start.handle,pend.handle,color.handle)
	EndMethod

	Method draw3DTriangle(triangle:Triangle3df,color:SColor=Null) ' def color=255,255,255,255
		If color=Null Then color=New SColor ' handle will be 0 tripping the default
		IrrVideo_IVideoDriver_draw3DTriangle(handle,triangle.handle,color.handle) 
	EndMethod
	
	Method draw3DBox(box:AABbox3df,color:SColor=Null) ' def color=255,255,255,255
		If color=Null Then color=New SColor ' handle will be 0 tripping the default
		IrrVideo_IVideoDriver_draw3DBox(handle,box.handle,color.handle)
	EndMethod
	
	Method draw2DImage(texture:ITexture,destPos:Position2di)
		IrrVideo_IVideoDriver_draw2DImage(handle,texture.handle,destPos.handle)
	EndMethod

	Method draw2DImageSegment(texture:ITexture,destPos:Position2di,sourceRect:Recti,clipRect:Recti=Null,color:SColor=Null,bUseAlphaChannelOfTexture:Int=False)
		IrrVideo_IVideoDriver_draw2DImageSegment(handle,texture.handle,destPos.handle,sourceRect.handle,_HANDLE(clipRect),_HANDLE(color),bUseAlphaChannelOfTexture)
	EndMethod

	Method draw2DImageSet(texture:ITexture,pos:Position2di,sourceRects:Recti[],indices:Int[],kerningWidth:Int=0,clipRect:Recti=Null,color:SColor=Null,bUseAlphaChannelOfTexture:Int=False)
		Local rectsArrSize:Int=0
		Local rectsArr:Int[]=GetHandleArray(sourceRects)

		IrrVideo_IVideoDriver_draw2DImageSet(handle,texture.handle,pos.handle,rectsArr,rectsArrSize,indices,indices.Dimensions()[0],kerningWidth,_HANDLE(clipRect),_HANDLE(color),bUseAlphaChannelOfTexture)
	EndMethod

	Method draw2DImageSegmentToRect(texture:ITexture,destRect:Recti,sourceRect:Recti,clipRect:Recti=Null,colors:SColor[]=Null,bUseAlphaChannelOfTexture:Int=False)
		If clipRect=Null Then clipRect=New Recti	' handle will be 0 tripping the default
		Local colors_size:Int=0
		Local colorsArr:Int[]=GetHandleArray(colors)
		If colorsArr Then colors_size=colorsArr.Dimensions()[0]

		IrrVideo_IVideoDriver_draw2DImageSegmentToRect(handle,texture.handle,destRect.handle,sourceRect.handle,clipRect.handle,colorsArr,colors_size,bUseAlphaChannelOfTexture)
	EndMethod

	Method draw2DRectangle(color:SColor,pos:Recti,clip:Recti=Null)
		If clip=Null Then clip=New Recti	' handle will be 0 tripping the default
		IrrVideo_IVideoDriver_draw2DRectangle(handle,color.handle,pos.handle,clip.handle)
	EndMethod

	Method draw2DRectangleWithGradient(pos:Recti,colorLeftUp:SColor,colorRightUp:SColor,colorLeftDown:SColor,colorRightDown:SColor,clip:Recti=Null)
		If clip=Null Then clip=New Recti	' handle will be 0 tripping the default
		IrrVideo_IVideoDriver_draw2DRectangleWithGradient(handle,pos.handle,colorLeftUp.handle,colorRightUp.handle,colorLeftDown.handle,colorRightDown.handle,clip.handle)
	EndMethod

	Method draw2DLine(start:Position2di,pEnd:Position2di,color:SColor=Null) ' def color=255,255,255,255
		If color=Null Then color=New SColor ' handle will be 0 tripping the default
		IrrVideo_IVideoDriver_draw2DLine(handle,start.handle,pEnd.handle,color.handle)
	EndMethod

	Method draw2DPolygon(center:Position2di,radius:Float,color:SColor=Null,vertexCount:Int=10) ' def color=100,255,255,255
		If color=Null Then color=New SColor ' handle will be 0 tripping the default
		IrrVideo_IVideoDriver_draw2DPolygon(handle,center.handle,radius,color.handle,vertexCount)
	EndMethod

	Method drawStencilShadowVolume(triangles:Array_triangle3df,count:Int,bZfail:Int=True)
		IrrVideo_IVideoDriver_drawStencilShadowVolume(handle,triangles.handle,count,bZfail)
	EndMethod
	
	Method drawStencilShadow(bClearStencilBuffer:Int=False,leftUpEdge:SColor=Null,rightUpEdge:SColor=Null,leftDownEdge:SColor=Null,rightDownEdge:SColor=Null) ' colors default to 255,0,0,0
		If leftUpEdge=Null Then leftUpEdge=New SColor ' handle will be 0 tripping the default
		If rightUpEdge=Null Then rightUpEdge=New SColor ' handle will be 0 tripping the default
		If leftDownEdge=Null Then leftDownEdge=New SColor ' handle will be 0 tripping the default
		If rightDownEdge=Null Then rightDownEdge=New SColor ' handle will be 0 tripping the default
		IrrVideo_IVideoDriver_drawStencilShadow(handle,bClearStencilBuffer,leftUpEdge.handle,rightUpEdge.handle,leftDownEdge.handle,rightDownEdge.handle)
	EndMethod

	Method drawMeshBuffer(mb:IMeshBuffer)
		IrrVideo_IVideoDriver_drawMeshBuffer(handle,mb.handle)
	EndMethod
	
	Method setFog(color:SColor=Null,bLinearFog:Int=True,start:Float=50.0,fend:Float=100.0,density:Float=0.01,bPixelFog:Int=False,bRangeFog:Int=False) ' color defaults to 0,255,255,255
		If color=Null Then color=New SColor ' handle will be 0 tripping the default
		IrrVideo_IVideoDriver_setFog(handle,color.handle,bLinearFog,start,fend,density,bPixelFog,bRangeFog)
	EndMethod
	
	Method getScreenSize:Dimension2di()
		Return Dimension2di.createFromHandle(IrrVideo_IVideoDriver_getScreenSize(handle))
	EndMethod
	
	Method getCurrentRenderTargetSize:Dimension2di()
		Return Dimension2di.createFromHandle(IrrVideo_IVideoDriver_getCurrentRenderTargetSize(handle),False)
	EndMethod

	
	Method getFPS:Int()
		Return IrrVideo_IVideoDriver_getFPS(handle)
	EndMethod
	
	Method getPrimitiveCountDrawn:Int(param:Int=0) ' returns unsigned 32
		Return IrrVideo_IVideoDriver_getPrimitiveCountDrawn(handle,param) 
	EndMethod
	
	Method deleteAllDynamicLights()
		IrrVideo_IVideoDriver_deleteAllDynamicLights(handle)
	EndMethod
	
	Method addDynamicLight(light:SLight)
		IrrVideo_IVideoDriver_addDynamicLight(handle,light.handle)
	EndMethod
	
	Method setAmbientLight(color:SColorf)
		IrrVideo_IVideoDriver_setAmbientLight(handle,color.handle)
	EndMethod
	
	Method getMaximalDynamicLightAmount:Int()
		Return IrrVideo_IVideoDriver_getMaximalDynamicLightAmount(handle)
	EndMethod
	
	Method getDynamicLightCount:Int()
		Return IrrVideo_IVideoDriver_getDynamicLightCount(handle)
	EndMethod
	
	Method getDynamicLight:SLight(idx:Int)
		Return SLight.createFromHandle(IrrVideo_IVideoDriver_getDynamicLight(handle,idx))
	EndMethod
	
	Method getName:String()
		 Return IrrVideo_IVideoDriver_getName(handle)
	EndMethod
	
	Method getMaximalPrimitiveCount:Int()
		Return IrrVideo_IVideoDriver_getMaximalPrimitiveCount(handle)
	EndMethod
	
	Method setTextureCreationFlag(E_TEXTURE_CREATION_FLAG:Int,bEnabled:Int)
		IrrVideo_IVideoDriver_setTextureCreationFlag(handle,E_TEXTURE_CREATION_FLAG,bEnabled)
	EndMethod
	
	Method getTextureCreationFlag:Int(E_TEXTURE_CREATION_FLAG:Int)
		Return IrrVideo_IVideoDriver_getTextureCreationFlag(handle,E_TEXTURE_CREATION_FLAG)
	EndMethod
	
	Method createImageFromFile:IImage(filename:String)
		Return IImage.createFromHandle(IrrVideo_IVideoDriver_createImageFromFile(handle,filename))
	EndMethod
	
	Method createImageFromStream:IImage(read:IReadFile)
		Return IImage.createFromHandle(IrrVideo_IVideoDriver_createImageFromStream(handle,read.handle))
	EndMethod
	
	Method createImageFromData:IImage(ECOLOR_FORMAT:Int,size:Dimension2di,pdata:Byte Ptr,ownForeignMemory:Int=False,deleteMemory:Int=True)
		Return IImage.createFromHandle(IrrVideo_IVideoDriver_createImageFromData(handle,ECOLOR_FORMAT,size.handle,pdata,ownForeignMemory,deleteMemory))
	EndMethod

	Method createRenderTargetTexture:ITexture(size:Dimension2di)
		Return ITexture.createFromHandle(IrrVideo_IVideoDriver_createRenderTargetTexture(handle,size.handle),True)
	EndMethod

	Method getMaterialRendererCount:Int()
		Return IrrVideo_IVideoDriver_getMaterialRendererCount(handle)
	EndMethod
	
	' dont Delete reference
	Method getMaterialRenderer:IMaterialRenderer(idx:Int)
		Return IMaterialRenderer.createFromHandle(IrrVideo_IVideoDriver_getMaterialRenderer(handle,idx),False)
	EndMethod 
	
	Method getExposedVideoData:SExposedVideoData()
		Return SExposedVideoData.createFromHandle(IrrVideo_IVideoDriver_getExposedVideoData(handle))
	EndMethod

	Method getDriverType:Int() ' returns E_DRIVER_TYPE	
		Return IrrVideo_IVideoDriver_getDriverType(handle) 
	EndMethod
	
	Method drawIndexedTriangleList(vertices:Array_S3DVertex,indexList:Array_u16)
		IrrVideo_IVideoDriver_drawIndexedTriangleList(handle,vertices.handle,indexList.handle)
	EndMethod 

	Method drawIndexedTriangleListFromCoords(vertices:Array_S3DVertex2TCoords,indexList:Array_u16)
		IrrVideo_IVideoDriver_drawIndexedTriangleListFromCoords(handle,vertices.handle,indexList.handle)
	EndMethod
	
	Method drawIndexedTriangleListTangents(vertices:Array_S3DVertexTangents,indexList:Array_u16)
		IrrVideo_IVideoDriver_drawIndexedTriangleListTangents(handle,vertices.handle,indexList.handle)
	EndMethod

	Method drawIndexedTriangleFan(vertices:Array_S3DVertex,indexList:Array_u16)
		IrrVideo_IVideoDriver_drawIndexedTriangleFan(handle,vertices.handle,indexList.handle)
	EndMethod 

	Method drawIndexedTriangleFanFromCoords(vertices:Array_S3DVertex2TCoords,indexList:Array_u16)
		IrrVideo_IVideoDriver_drawIndexedTriangleFanFromCoords(handle,vertices.handle,indexList.handle)
	EndMethod 
	
	Method clearZBuffer()
		IrrVideo_IVideoDriver_clearZBuffer(handle)
	EndMethod
	
	Method getTextureByIndex:ITexture(index:Int)
		Return ITexture.createFromHandle(IrrVideo_IVideoDriver_getTextureByIndex(handle,index),False)
	EndMethod
	
	Method getTextureCount:Int()
		Return IrrVideo_IVideoDriver_getTextureCount(handle)
	EndMethod

	Method addMaterialRenderer:Int(renderer:IMaterialRenderer,name:String=Null)
		Return IrrVideo_IVideoDriver_addMaterialRenderer(handle,renderer.handle,name)
	EndMethod

	Method addExternalImageLoader(loader:IImageLoader)
		IrrVideo_IVideoDriver_addExternalImageLoader(handle,loader.handle)
	EndMethod
	
	Method addExternalImageWriter(writer:IImageWriter)
		IrrVideo_IVideoDriver_addExternalImageWriter(handle,writer.handle)
	EndMethod
	
	Method writeImageToFile:Int(image:IImage,filename:String,param:Int=0)
		Return IrrVideo_IVideoDriver_writeImageToFile(handle,image.handle,filename,param)
	EndMethod
	
	Method getMaterialRendererName:String(idx:Int)
		Return IrrVideo_IVideoDriver_getMaterialRendererName(handle,idx)
	EndMethod
	
	Method setMaterialRendererName(idx:Int,name:String)
		IrrVideo_IVideoDriver_setMaterialRendererName(handle,idx,name)
	EndMethod
	
	Method createAttributesFromMaterial:IAttributes(material:SMaterial)
		Return IAttributes.createFromHandle(IrrVideo_IVideoDriver_createAttributesFromMaterial(handle,material.handle),True)
	EndMethod

	Method fillMaterialStructureFromAttributes(out:SMaterial,attributes:IAttributes)
		IrrVideo_IVideoDriver_fillMaterialStructureFromAttributes(handle,out.handle,attributes.handle)
	EndMethod
	
	Method createScreenShot:IImage()
		Return IImage.createFromHandle(IrrVideo_IVideoDriver_createScreenShot(handle),True)
	EndMethod

	Method renameTexture(texture:ITexture,newName:String)
		IrrVideo_IVideoDriver_renameTexture(handle,texture.handle,newName)
	EndMethod

	Method drop:Int() Final
		If mustdelete Then mustdelete=False		
		Return IrrVideo_IVideoDriver_drop(handle)
	EndMethod
	
	Method getDebugName:String() Final
		Return IrrVideo_IVideoDriver_getDebugName(handle)
	EndMethod
	
	Method grab() Final
		IrrVideo_IVideoDriver_grab(handle)
	EndMethod
	
	Method Delete()
		If (mustdelete) Then drop()
	EndMethod	
EndType
